/*
 *  The Sun Project JXTA(TM) Software License
 *  
 *  Copyright (c) 2001-2006 Sun Microsystems, Inc. All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions are met:
 *  
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  
 *  2. Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *  
 *  3. The end-user documentation included with the redistribution, if any, must 
 *     include the following acknowledgment: "This product includes software 
 *     developed by Sun Microsystems, Inc. for JXTA(TM) technology." 
 *     Alternately, this acknowledgment may appear in the software itself, if 
 *     and wherever such third-party acknowledgments normally appear.
 *  
 *  4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must 
 *     not be used to endorse or promote products derived from this software 
 *     without prior written permission. For written permission, please contact 
 *     Project JXTA at https://jxta.dev.java.net.
 *  
 *  5. Products derived from this software may not be called "JXTA", nor may 
 *     "JXTA" appear in their name, without prior written permission of Sun.
 *  
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SUN 
 *  MICROSYSTEMS OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 *  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 *  JXTA is a registered trademark of Sun Microsystems, Inc. in the United 
 *  States and other countries.
 *  
 *  Please see the license information page at :
 *  <https://jxta.dev.java.net/license.html> for instructions on use of 
 *  the license in source files.
 *  
 *  ====================================================================

 *  This software consists of voluntary contributions made by many individuals 
 *  on behalf of Project JXTA. For more information on Project JXTA, please see 
 *  https://jxta.dev.java.net/
 *  
 *  This license is based on the BSD license adopted by the Apache Foundation. 
 *  
 */

package net.jxta.impl.endpoint.transportMeter;

import java.util.ResourceBundle;

/**
 * This class is used to extract runtime transport metering code configuration.
 * 
 */
public class ConditionalTransportMeterBuildSettings {

    /**
     * This method indicates whether the transport metering code should be executed or not.
     * 
     * <p>It searches a {@code net.jxta.meter.conditionalTransportMetering} property entry in
     * the {@code net.jxta.user} property file included as part of the generated .jar file.
     * If this property is set to ON, the returned value will be {@code true}. Else, if the
     * property is not defined or if it has another value, this method will return {@false}.
     * 
     * @return {@code true} if tranport metering code should be executed, else {@code false}
     */
    public static boolean isRuntimeMetering() {

        boolean runtimeMetering = false;

        try {
                ResourceBundle userResourceBundle = ResourceBundle.getBundle( "net.jxta.user" );
                String meteringProperty = "net.jxta.meter.conditionalTransportMetering";
                String meteringValue = userResourceBundle.getString( meteringProperty );
                runtimeMetering = "on".equalsIgnoreCase( meteringValue );
        } catch (Exception ignored) {
        }

        return runtimeMetering;

    }

}
